@@ -8,14 +8,25 @@ ADDRESS_COMMAND_1 = 0x4070
 ADDRESS_MA_MIN_VALUE = 0x4086
 ADDRESS_MA_MAX_VALUE = 0x4087
 ADDRESS_CURRENT_MODE = 0x407b
+ADDRESS_LCD_WRITE_PARAMETER_1 = 0x4180
+ADDRESS_LCD_WRITE_PARAMETER_2 = 0x4181
 ADDRESS_POWER_LEVEL = 0x41f4
 ADDRESS_LEVELMA = 0x420D
 ADDRESS_KEY = 0x4213
 
+# EEPROM addresses
+EEPROM_ADDRESS_POWER_LEVEL = 0x8009
+EEPROM_ADDRESS_FAVORITE_MODE = 0x800C
+
 # Commands
 COMMAND_START_FAVORITE_MODULE = 0x00
+COMMAND_SHOW_STATUS_SCREEN = 0x02
+COMMAND_SELECT_MENU_ITEM = 0x03
 COMMAND_EXIT_MENU = 0x04
 COMMAND_NEW_MODE = 0x12
+COMMAND_WRITE_CHARACTER_TO_LCD = 0x13
+COMMAND_WRITE_STRING_TO_LCD = 0x15
+COMMAND_NO_COMMAND = 0xff
 
 # Modes
 MODE_POWERON = 0x00
